from .extractors import eztv, yts, find_similar
import os
from halo import Halo


class Ezflix(object):
    def __init__(
        self,
        query,
        media_type="tv",
        limit=20,
        sort_by="seeds",
        sort_order="desc",
        quality=None,
        minimum_rating=None,
        language="en",
        page=1,
        debug=False,
        cli_mode=False,
    ):
        self._torrents = []
        self._query = query
        self._media_type = media_type
        self._limit = limit
        self._sort_by = sort_by
        self._sort_order = sort_order
        self._quality = quality
        self._minimum_rating = minimum_rating
        self._language = language
        self._page = page
        self._debug = debug
        self._cli_mode = cli_mode

    def torrent_info(self, val):
        if self._torrents is None or int(val) > len(self._torrents):
            return None
        for torrent in self._torrents:
            if torrent["id"] == int(val):
                return torrent
        return None

    def search_subtitles(self, media_title):
        os.system("subliminal download -l %s '%s'" % (self._language, media_title))
        cur_dir = os.getcwd()
        file_list = os.listdir(cur_dir)
        for f in file_list:
            if media_title in f:
                return f

        return None

    def similar(self, t_id):
        spinner = None
        if self._cli_mode:
            spinner = Halo(text="Searching...", spinner="dots")
            spinner.start()
        if self._media_type == "movie":
            torrent_info = self.torrent_info(t_id)
            if torrent_info is None:
                return
            self._torrents = find_similar(
                torrent_info["yts_id"], self._quality, self._limit
            )
        if self._cli_mode:
            spinner.stop()
        return self._torrents

    def search(self):
        spinner = None
        if self._cli_mode:
            spinner = Halo(text="Searching...", spinner="dots")
            spinner.start()
        if self._media_type == "tv":
            self._torrents = eztv(
                self._query.replace(" ", "-").lower(),
                page=self._page,
                limit=self._limit,
                quality=self._quality,
                debug=self._debug,
            )
        elif self._media_type == "movie":
            self._torrents = yts(
                q=self._query,
                limit=self._limit,
                sort_by=self._sort_by,
                sort_order=self._sort_order,
                quality=self._quality,
                minimum_rating=self._minimum_rating,
                page=self._page,
                debug=self._debug,
            )
        if self._cli_mode:
            spinner.stop()
        return self._torrents



from .utils import cmd_exists, peerflix
import sys
import colorful
from .argument_parser import EzflixParser
from .ezflix import Ezflix
from prettytable import PrettyTable
from builtins import input
import time
import subprocess

if sys.version_info < (2, 7):
    sys.exit("You need Python 2.7 or later\n")

parser = EzflixParser()

if len(sys.argv) == 1:
    parser.error()
    sys.exit(0)

args = parser.parse()

media_player = args.media_player
debug = args.debug

if not cmd_exists("peerflix"):
    sys.exit("This program requires Peerflix. https://github.com/mafintosh/peerflix")

if not cmd_exists("mpv") and args.media_player == "mpv":
    print(
        colorful.bold(
            "In ezflix, the default player is mpv. It will fallback to vlc if mpv isn't found. "
            "You can use the media_player argument to set your media player."
        )
    )
    media_player = "vlc"

if not (args.query and args.query.strip()):
    sys.exit(colorful.red("Search query not valid."))


def main():
    p = 1
    search(p)
    if not args.no_menu:
        display_menu()
    while True:
        try:
            i = input()
        except KeyboardInterrupt:
            continue
        except EOFError:
            break
        if i == "quit":
            sys.exit()
        elif i == "search":
            refined_query = input("Enter search query: ")
            search(page=1, term=refined_query)
            continue
        elif i == "next":
            p += 1
            search(p)
            continue
        elif i == "prev":
            if p > 1:
                p -= 1
                search(p)
            continue
        elif "info" in i:
            media_info(i)
            continue
        elif "similar" in i:
            similar(i)
            continue
        elif "trailer" in i:
            play_trailer(i)
            continue
        elif "imdb" in i:
            open_imdb_page(i)
            continue
        if not str.isdigit(i):
            print(colorful.red("Invalid selection."))
            continue
        magnet = ezflix.torrent_info(int(i))
        if not magnet:
            print(colorful.red("Invalid selection."))
            continue
        print("Playing " + magnet["title"])
        file_path = ezflix.search_subtitles(magnet["title"]) if args.subtitles else None
        peerflix(magnet["link"], media_player, args.subtitles, args.remove, file_path)
        sys.exit()


def display_menu():
    pt = PrettyTable()
    pt.field_names = [
        "Help Menu : Pass --no-menu if you don't want to display this menu on program startup."
    ]
    pt.align = "l"
    pt.add_row(["Enter the ID of the media you want to stream."])
    pt.add_row(["Enter 'next' to see the next page of movies."])
    pt.add_row(["Enter 'prev' to see the previous page of movies."])
    pt.add_row(["Enter 'search' to refine your search."])
    pt.add_row(
        ["Enter 'info' and the id of the torrent to get the movie/tv show overview."]
    )
    pt.add_row(["Enter 'trailer' and the id of the torrent to play the movie trailer."])
    pt.add_row(["Enter 'similar' and the id of the torrent to find similar movies."])
    pt.add_row(
        ["Enter 'imdb' and the id of the torrent to open the imdb media webpage."]
    )
    pt.add_row(["Enter 'quit' to close the program."])
    print(pt)


def display_table(torrents):
    pt = PrettyTable()
    pt.field_names = ["Id", "Torrent", "Seeds", "Peers", "Released", "Rating", "Genre"]
    pt.align = "l"
    t = 0
    for result in torrents:
        if (
            not (result["seeds"] == 0 or result["seeds"] is None)
            or args.no_seeds is True
        ):
            pt.add_row(
                [
                    result["id"],
                    result["title"],
                    result["seeds"],
                    result["peers"],
                    result["release_date"],
                    result["rating"],
                    result["genre"],
                ]
            )
            t += 1
    if t > 0 or args.no_seeds:
        print(pt)
    else:
        print(colorful.red("No results found."))
        sys.exit(0)


def search(page=1, term=None):
    global ezflix
    ezflix = Ezflix(
        query=args.query if term is None else term,
        media_type=args.media_type,
        limit=int(args.limit),
        sort_by=args.sort_by,
        sort_order=args.sort_order,
        quality=args.quality,
        minimum_rating=args.minimum_rating,
        language=args.language,
        page=page,
        debug=debug,
        cli_mode=True,
    )
    torrents = ezflix.search()
    if torrents is None or len(torrents) == 0:
        sys.exit(colorful.red("No results found."))
    if args.latest:
        latest = torrents[0]
        file_path = ezflix.search_subtitles(latest["title"]) if args.subtitles else None
        print("Playing " + latest["title"])
        time.sleep(2.5)
        peerflix(
            magnet_link=latest["link"],
            media_player=media_player,
            subtitles=args.subtitles,
            remove=args.remove,
            file_path=file_path,
        )
        sys.exit()
    display_table(torrents)


def media_info(user_input):
    sp = user_input.split()
    if not len(sp) > 1 or not str.isdigit(sp[1]):
        print(
            colorful.red(
                "Incorrect usage. Enter 'info' and the id of the torrent to get the movie/tv show overview."
            )
        )
        return
    torrent_info = ezflix.torrent_info(sp[1])
    if torrent_info is None:
        print(colorful.red("Invalid selection."))
        return
    print(torrent_info["overview"])


def similar(user_input):
    if args.media_type == "tv":
        print(colorful.red("Currently, movie torrents are only supported."))
        return
    sp = user_input.split()
    if not len(sp) > 1 or not str.isdigit(sp[1]):
        print(
            colorful.red(
                "Incorrect usage. Enter 'similar' and the id of the torrent to find similar movies."
            )
        )
        return
    torrents = ezflix.similar(sp[1])
    display_table(torrents)


def play_trailer(user_input):
    if args.media_type == "tv":
        print(colorful.red("Currently, movie torrents are only supported."))
        return
    sp = user_input.split()
    if not len(sp) > 1 or not str.isdigit(sp[1]):
        print(
            colorful.red(
                "Incorrect usage. Enter 'trailer' and the id of the torrent to play the movie trailer."
            )
        )
        return
    torrent_info = ezflix.torrent_info(sp[1])
    if torrent_info is None:
        print(colorful.red("Invalid selection."))
        return
    trailer = torrent_info["trailer"]
    if not trailer:
        print(colorful.red("No trailer found."))
        return
    geometry = "50%"
    subprocess.Popen(
        [
            "/bin/bash",
            "-c",
            "%s https://www.youtube.com/watch?v=%s --geometry=%s"
            % (media_player, trailer, geometry),
        ]
    )


def open_imdb_page(user_input):
    sp = user_input.split()
    if not len(sp) > 1 or not str.isdigit(sp[1]):
        print(
            colorful.red(
                "Incorrect usage. Enter 'imdb' and the id of the torrent to open the imdb page for the movie."
            )
        )
        return
    torrent_info = ezflix.torrent_info(sp[1])
    if torrent_info is None:
        print(colorful.red("Invalid selection."))
        return
    imdb = torrent_info["imdb"]
    if not imdb:
        print(colorful.red("No imdb found."))
        return
    subprocess.Popen(
        ["/bin/bash", "-c", "open https://www.imdb.com/title/%s/" % imdb]
    )


if __name__ == "__main__":
    main()



import subprocess


def cmd_exists(cmd):
    return (
        subprocess.call(
            "type " + cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        == 0
    )


def peerflix(magnet_link, media_player, subtitles, remove, file_path):
    subtitles = (
        "--subtitles '%s'" % file_path if subtitles and file_path is not None else ""
    )
    remove = "--remove" if remove else ""
    cmd = 'peerflix "%s" --%s %s %s' % (magnet_link, media_player, subtitles, remove)
    print("Executing " + cmd)
    subprocess.Popen(["/bin/bash", "-c", cmd])



import requests
from datetime import datetime
from tmdbv3api import TMDb, TV
from pprint import pprint
from ezflix.torrent import Torrent

tmdb = TMDb()
tmdb.api_key = "e1076b74406e0a7d0efb5318f1b662d0"
URL = "https://eztv.ag/api/get-torrents"


def search_tmdb_shows(q):
    tv = TV()
    sr = tv.search(q)
    return sr


def get_external_id(show_id):
    tv = TV()
    details = tv.external_ids(show_id)
    return details.imdb_id if hasattr(details, "imdb_id") else None


def eztv(q, limit, page=1, quality=None, debug=False):
    sr = search_tmdb_shows(q)
    if not sr:
        return
    overview = sr[0].overview
    imdb_id = get_external_id(sr[0].id)[2:]
    if imdb_id:
        req = requests.get(
            "%s?imdb_id=%s&page=%s&limit=%s" % (URL, imdb_id, page, limit)
        )
        if debug:
            print(req.status_code)
        results, count = [], 1
        search_results = req.json()
        if debug:
            pprint(search_results)
        if "torrents" in search_results:
            for result in search_results["torrents"]:
                release_date = datetime.fromtimestamp(int(result["date_released_unix"]))
                torrent = Torrent(
                    count,
                    result["title"],
                    result["magnet_url"],
                    result["seeds"],
                    result["peers"],
                    overview,
                    "-",
                    imdb_id,
                    release_date,
                    genre="-",
                )
                torrent_dict = torrent.build()
                if quality is not None:
                    if quality.lower() in result["title"]:
                        results.append(torrent_dict)
                        count += 1
                else:
                    results.append(torrent_dict)
                    count += 1
        if debug:
            pprint(results)
        return results



# -*- coding: utf-8 -*-

from argparse import ArgumentParser
import os
import sys

HERE = os.path.abspath(os.path.dirname(__file__))

version_ns = {}

with open(os.path.join(HERE, "__version__.py")) as f:
    exec(f.read(), {}, version_ns)


class EzflixParser(object):
    def __init__(self):
        self._parser = ArgumentParser(prog="ezflix")
        self._supported_players = [
            "mpv",
            "vlc",
            "mplayer",
            "smplayer",
            "mpchc",
            "potplayer",
            "webplay",
            "omx",
            "airplay",
        ]

        self._sort_types = [
            "download_count",
            "like_count",
            "date_added",
            "seeds",
            "peers",
            "rating",
            "title",
            "year",
        ]

        self._sort_orders = ["asc", "desc"]

        self._media_types = ["movie", "tv"]

        self._qualities = ["720p", "1080p", "3d"]
        self._add_arguments()

    def _add_arguments(self):
        self._parser.add_argument(
            "media_type",
            help="The media type.",
            default="tv",
            nargs="?",
            choices=self._media_types,
        )

        self._parser.add_argument("query", help="The search query.")

        self._parser.add_argument(
            "--limit",
            help="TThe limit of results per page that has been set",
            default="20",
            nargs="?",
        )

        self._parser.add_argument(
            "--minimum_rating",
            help="Used to filter movie by a given minimum IMDb rating",
            nargs="?",
        )

        self._parser.add_argument(
            "--media_player",
            help="The media player.",
            default="mpv",
            nargs="?",
            choices=self._supported_players,
        )

        self._parser.add_argument(
            "--latest",
            help="Play the latest TV episode.",
            dest="latest",
            action="store_true",
        )

        self._parser.add_argument(
            "--subtitles",
            help="Load subtitles file.",
            dest="subtitles",
            action="store_true",
        )

        self._parser.add_argument(
            "--sort_by",
            help="Sorts the results by choosen value",
            default="date_added",
            nargs="?",
            choices=self._sort_types,
        )

        self._parser.add_argument(
            "--sort_order",
            help="Orders the results by either Ascending or Descending order",
            default="desc",
            nargs="?",
            choices=self._sort_orders,
        )

        self._parser.add_argument(
            "--quality",
            help="Used to filter by a given quality.",
            nargs="?",
            choices=self._qualities,
        )

        self._parser.add_argument(
            "--genre",
            help="Used to filter by a given genre (See http://www.imdb.com/genre/ for full list)",
        )

        self._parser.add_argument(
            "--remove",
            help="Remove files on exit.",
            default=True,
            dest="remove",
            action="store_true",
        )

        self._parser.add_argument(
            "--no_seeds",
            help="Include torrents that have no seeds",
            default=False,
            dest="no_seeds",
            action="store_true",
        )

        self._parser.add_argument(
            "--debug",
            help="Set this flag to print JSON to stdout.",
            default=False,
            dest="debug",
            action="store_true",
        )

        self._parser.add_argument(
            "--no-menu",
            help="Set this flag to not show the usage menu on program startup.",
            default=False,
            dest="no_menu",
            action="store_true",
        )

        self._parser.add_argument(
            "--language",
            help="Language as IETF code. Set this argument to download subtitles in a given language.",
            default="en",
        )

        self._parser.add_argument(
            "-v",
            "--version",
            action="version",
            version="%(prog)s (" + version_ns["__version__"] + ")",
        )

    def error(self):
        print(
            """              
                ███████╗███████╗███████╗██╗     ██╗██╗  ██╗
                ██╔════╝╚══███╔╝██╔════╝██║     ██║╚██╗██╔╝
                █████╗    ███╔╝ █████╗  ██║     ██║ ╚███╔╝ 
                ██╔══╝   ███╔╝  ██╔══╝  ██║     ██║ ██╔██╗ 
                ███████╗███████╗██║     ███████╗██║██╔╝ ██╗
                ╚══════╝╚══════╝╚═╝     ╚══════╝╚═╝╚═╝  ╚═╝                                                                                        
        """
        )
        print("Ezflix v" + version_ns["__version__"])
        self._parser.print_help()
        sys.exit(0)

    def parse(self):
        return self._parser.parse_args()



import requests
from pprint import pprint
from ezflix.torrent import Torrent

URL = "https://yts.ag/api/v2"


def _build_obj(req, quality, limit):
    if (
        "movies" not in req["data"]
        or not req["status"] == "ok"
        or not req["data"]["movie_count"] > 0
    ):
        return
    arr, count = [], 1
    for r in req["data"]["movies"]:
        if "torrents" in r:
            for torrent in r["torrents"]:
                title = "%s (%s) (%s)" % (r["title"], r["year"], torrent["quality"])
                release_date = torrent["date_uploaded"]
                info = Torrent(
                    count,
                    title,
                    torrent["url"],
                    torrent["seeds"],
                    torrent["peers"],
                    r["synopsis"],
                    r["rating"],
                    r["imdb_code"],
                    release_date,
                    trailer=r["yt_trailer_code"],
                    yts_id=r["id"],
                    genre=r["genres"][0],
                )
                torrent_dict = info.build()
                if quality is not None:
                    if quality == torrent["quality"]:
                        arr.append(torrent_dict)
                        count += 1
                else:
                    arr.append(torrent_dict)
                    count += 1

    return arr[0:limit]


def find_similar(movie_id, quality=None, limit=20, debug=False):
    return make_request(
        "%s/movie_suggestions.json" % URL, quality, limit, {"movie_id": movie_id}, debug
    )


def yts(
    q,
    quality=None,
    limit=20,
    minimum_rating=4,
    sort_by="date_added",
    sort_order="asc",
    page=1,
    debug=False,
):
    params = {
        "query_term": q,
        "sort_by": sort_by,
        "sort_order": sort_order,
        "limit": limit,
        "quality": quality,
        "minimum_rating": minimum_rating,
        "page": page,
    }
    return make_request("%s/list_movies.json" % URL, quality, limit, params, debug)


def make_request(url, quality, limit, params, debug):
    req = requests.get(url, params=params)
    req = req.json()
    if debug:
        pprint(req)
    return _build_obj(req, quality, limit)



class Torrent:
    def __init__(
        self,
        torrent_id,
        title,
        link,
        seeds,
        peers,
        overview,
        rating,
        imdb,
        release_date,
        **kwargs
    ):
        self.torrent_id = torrent_id
        self.title = title
        self.link = link
        self.seeds = seeds
        self.peers = peers
        self.overview = overview
        self.rating = rating
        self.imdb = imdb
        self.release_date = release_date
        self.kwargs = kwargs

    def build(self):
        base = {
            "id": self.torrent_id,
            "title": self.title,
            "link": self.link,
            "seeds": self.seeds,
            "peers": self.peers,
            "overview": self.overview,
            "rating": self.rating,
            "imdb": self.imdb,
            "release_date": self.release_date,
        }

        torrent = base.copy()
        torrent.update(self.kwargs)
        return torrent
